/*
    Copyright (C) 2002 Paul Davis 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: automation_event.h,v 1.6 2003/02/21 19:01:31 pbd Exp $
*/

#ifndef __ardour_automation_event_h__
#define __ardour_automation_event_h__

#include <vector>
#include <sigc++/signal_system.h>
#include <pbd/lockmonitor.h>
#include <ardour/ardour.h>

using std::vector;

namespace ARDOUR {
	
struct AutomationEvent {
    unsigned long what;
    jack_nframes_t when;
    float value;
    
    AutomationEvent (unsigned long p, float v, jack_nframes_t w) 
	    : what (p), when (w), value (v) { }
    
    bool operator==(const AutomationEvent& other) {
	    return what == other.what && value == other.value && when == other.when;
    }
};

class AutomationList : public SigC::Object, public vector<AutomationEvent>
{
  public:
	AutomationList();
	~AutomationList() {}

	SigC::Signal0<void> Changed;

	void freeze();
	void thaw ();
	void clear ();
	void sort();

	void push_back (const AutomationEvent& ev);
	void erase_range (unsigned long what, jack_nframes_t start, jack_nframes_t end);

	bool control_points_adjacent (double xval, size_t& before, size_t& after, double& before_x, double& after_x);

	template<class T> void apply_to_points (T& obj, void (T::*method)(const AutomationList&)) {
		LockMonitor lm (lock, __LINE__, __FILE__);
		(obj.*method)(*this);
	}

  private:
	PBD::Lock lock;
	bool _frozen;
	bool changed_when_thawed;
};

} // namespace

#endif /* __ardour_automation_event_h__ */
