/*
    Copyright (C) 1999 Paul Davis 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: configuration.h,v 1.20 2003/02/06 06:13:24 pbd Exp $
*/

#ifndef __ardour_configuration_h__
#define __ardour_configuration_h__

#include <map>

#include <sys/types.h>
#include <string>
#include <glib.h>

#include <ardour/stateful.h>

using std::string;

class XMLNode;

namespace ARDOUR {

class Configuration : public Stateful
{
  public:
	Configuration();
	virtual ~Configuration();

	struct MidiPortDescriptor {
	    string tag;
	    string device;
	    string type;
	    string mode;

	    MidiPortDescriptor (const XMLNode&);
	    XMLNode& get_state();
	};

	std::map<string,MidiPortDescriptor *> midi_ports;

	int load_state ();
	int save_state ();

	XMLNode& option_node (const string &, const string &);
	
	int set_state (const XMLNode&);
	XMLNode& get_state (void);

	XMLNode& get_keys() const;
	void set_keys(XMLNode&);

	string get_pixmap_path();
	void set_pixmap_path(string);

	string get_session_path();
	string get_top_session_path();
	void set_session_path(string);

	unsigned long get_minimum_disk_io(); 
	void set_minimum_disk_io(unsigned long);

	float get_track_buffer();
	void set_track_buffer(float);

	string get_ui_rcfile();
	void set_ui_rcfile(string);

	string get_soundfile_db();
	void set_soundfile_db(string);

	bool sequencer_thread();
	void set_sequencer_thread(bool);

	bool sequencer_thread_real();
	void set_sequencer_thread_real(bool);

	int get_sequencer_thread_priority();
	void set_sequencer_thread_priority(int);

	int get_sequencer_thread_interval();
	void set_sequencer_thread_interval(int);

	bool does_hiding_groups_deactivates_groups();
	void set_hiding_groups_deactivates_groups(bool);

	string get_auditioner_output_left();
	void set_auditioner_output_left(string);

	string get_auditioner_output_right();
	void set_auditioner_output_right(string);

	bool get_mute_affects_pre_fader();
	void set_mute_affects_pre_fader (bool);

	bool get_mute_affects_post_fader();
	void set_mute_affects_post_fader (bool);

	bool get_mute_affects_control_outs ();
	void set_mute_affects_control_outs (bool);

	bool get_mute_affects_main_outs ();
	void set_mute_affects_main_outs (bool);

	bool get_solo_latch ();
	void set_solo_latch (bool);

	unsigned long get_disk_choice_space_threshold();
	void set_disk_choice_space_threshold (unsigned long);

	string get_mmc_port_name();
	void   set_mmc_port_name(string);

	string get_mtc_port_name();
	void   set_mtc_port_name(string);

	bool get_monitor_inputs();
	void set_monitor_inputs(bool);

	bool get_jack_time_master();
	void set_jack_time_master(bool);

	bool get_native_format_is_bwf();
	void set_native_format_is_bwf(bool);

  private:
	void set_defaults ();
	const char* get_user_path();
	const char* get_system_path();
	
	string pixmap_path;
	string session_path;
	unsigned long minimum_disk_io_bytes;
	float  track_buffer_seconds;
	string ui_rcfile;
	string soundfile_db;
	bool   run_sequencer_thread;
	bool   sequencer_thread_is_real_time;
	int    sequencer_thread_priority;
	int    sequencer_thread_interval;
	bool   hiding_groups_deactivates_groups;
	string auditioner_output_left;
	string auditioner_output_right;
	bool	mute_affects_pre_fader;
	bool	mute_affects_post_fader;
	bool	mute_affects_control_outs;
	bool	mute_affects_main_outs;
	bool	solo_latch;
	unsigned long disk_choice_space_threshold;
	string  mtc_port_name;
	string  mmc_port_name;
	bool    monitor_inputs;
	bool    be_jack_time_master;
	bool    native_format_is_bwf;
	
	XMLNode *key_node;
};

extern Configuration *Config;

}; /* namespace ARDOUR */

#endif /* __ardour_configuration_h__ */

