/*
    Copyright (C) 2001 Paul Davis 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: newcurve.h,v 1.2 2002/04/10 17:42:58 pbd Exp $
*/

#ifndef __ardour_curve_h__
#define __ardour_curve_h__

#include <vector>
#include <pthread.h>
#include <sigc++/signal_system.h>

#include <ardour/spline.h>

namespace ARDOUR {

struct ControlPoint : public SplinePoint {
    enum Type {
	    XFadeIn,
	    XFadeOut,
	    DeclickIn,
	    DeclickOut,
	    Automation,
	    RegionGain
    };
    Type type;
};

class Curve : public SigC::Object
{
  public:
	Curve ();
	Curve (const Curve&);
	~Curve ();
	Curve& operator= (const Curve&);

	unsigned int n_control_points() const { return control_points.size(); }
	ControlPoint& operator[] (unsigned int n);

	void clear ();

	void add_control_point (ControlPoint&);
	void add_control_points (vector<ControlPoint>&);
	void remove_control_point (unsigned long);
	void reset_control_point (unsigned long, ControlPoint& newp);

	void hide (ControlPoint::Type);
	void show (ControlPoint::Type);

	void add_to_spline (Spline *spl);
	void set_spline (Spline *spl);

	SigC::Signal0<void> PointChanged;
	SigC::Signal0<void> PointAdded;
	SigC::Signal0<void> PointRemoved;
	SigC::Signal0<void> Cleared;

  private:
	vector<ControlPoint> control_points;
	vector<ControlPoint> hidden_points;
	PBD::Lock            lock;

	void hide_internal (ControlPoint::Type);
	void show_internal (ControlPoint::Type);
};

}; /* namespace ARDOUR */

#endif /* __ardour_gain_h__ */


