/*
    Copyright (C) 2002 Paul Davis 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: types.h,v 1.4 2003/02/10 14:31:56 pbd Exp $
*/

#ifndef __ardour_types_h__
#define __ardour_types_h__

#include <jack/types.h>

#if __GNUC__ < 3
typedef int intptr_t;
#endif

namespace ARDOUR {

	typedef jack_default_audio_sample_t      Sample;
	typedef float  pan_t;
	typedef float  gain_t;

	enum AutomationType {
		GainAutomation = 0x1,
		PanAutomation = 0x2,
		PluginAutomation = 0x4,
		SoloAutomation = 0x8,
		MuteAutomation = 0x10,
	};

	typedef unsigned char layer_t;
	typedef unsigned long long id_t;

	struct SMPTE_Time {
	    bool negative;
	    long hours;
	    long minutes;
	    long seconds;
	    long frames;
	    long subframes; // mostly not used

	    SMPTE_Time() {
		    negative = false;
		    hours = 0;
		    minutes = 0;
		    seconds = 0;
		    frames = 0;
		    subframes = 0;
	    }
			   
	};

	struct BBT_Time {
	    unsigned long bars;
	    unsigned long beats;
	    unsigned long ticks;

	    BBT_Time() {
		    bars = 1;
		    beats = 1;
		    ticks = 0;
	    }

	    /* we can't define arithmetic operators for BBT_Time, because
	       the results depend on a TempoMap, but we can define 
	       a useful check on the less-than condition.
	    */

	    bool operator< (const BBT_Time& other) const {
		    return bars < other.bars || 
			    (bars == other.bars && beats < other.beats) ||
			    (bars == other.bars && beats == other.beats && ticks < other.ticks);
	    }

	    bool operator== (const BBT_Time& other) const {
		    return bars == other.bars && beats == other.beats && ticks == other.ticks;
	    }
	    
	};

	struct AudioRange {
	    jack_nframes_t start;
	    jack_nframes_t end;
	    unsigned long id;
	    
	    AudioRange (jack_nframes_t s, jack_nframes_t e, unsigned long i) : start (s), end (e) , id (i) {}
	    
	    jack_nframes_t length() { return end - start + 1; } 

	    bool operator== (const AudioRange& other) const {
		    return start == other.start && end == other.end && id == other.id;
	    }

	    bool equal (const AudioRange& other) const {
		    return start == other.start && end == other.end;
	    }
	};
	
	struct MusicRange {
	    BBT_Time start;
	    BBT_Time end;
	    unsigned long id;
	    
	    MusicRange (BBT_Time& s, BBT_Time& e, unsigned long i)
		    : start (s), end (e), id (i) {}

	    bool operator== (const MusicRange& other) const {
		    return start == other.start && end == other.end && id == other.id;
	    }

	    bool equal (const MusicRange& other) const {
		    return start == other.start && end == other.end;
	    }
	};
};

#endif /* __ardour_types_h__ */

